/*
 * conf.h
 * Copyright (C) 2000 Nedelcho Stanev
 *
 * $Id: conf.h,v 1.00 2000/03/03 22:17:23 decho Exp $
 */

#ifndef INCLUDED_conf_h
#define INCLUDED_conf_h

#ifndef INCLUDED_sys_types_h
#include <sys/types.h>         /* time_t */
#define INCLUDED_sys_types_h
#endif

#ifndef INCLUDED_config_h
#include "config.h"
#define INCLUDED_config_h
#endif

struct Server;

/* server list flags */
#define SL_DELETE     (1 << 0) /* to delete servers after a rehash */
#define SL_CONNECT    (1 << 1) /* currently connecting to this server */

#define SetHubConnect(x)    ((x)->flags |= SL_CONNECT)
#define IsHubConnect(x)     ((x)->flags &  SL_CONNECT)
#define ClearHubConnect(x)  ((x)->flags &= ~SL_CONNECT)

/* Port flags */
#define PRT_TCM          1 /* accept only tcm bot connections */
#define PRT_USERS        2 /* accept only user connections */
#define PRT_DELETE      3 /* marked for deletion in a rehash */

struct MyInfo
{
  char *name;     /* services' server name */
  char *info;     /* services' info line */
  char *admin;    /* administrative info */
  struct Server *sptr; /* pointer to services entry in server list */
  struct Server *hub;  /* pointer to services' current hub server */
  
  struct Luser *osptr;   /* pointer to OperServ client structure */

};

/* Stores server S: lines from sockscleaner.conf */
struct Servlist
{
  struct Servlist *next;
  char *hostname;       /* hostname of hub server */
  char *password;       /* password for connection */
  char *realname;       /* network name of hub (if differnet from hostname) */
  int port;             /* port for connection */
  time_t connect_ts;    /* time we connected to it */
  int sockfd;           /* socket fd (must match HubSock) */
  unsigned int flags;
};

/* Stores P: line info from sockscleaner.conf */
struct PortInfo
{
  struct PortInfo *next;
  int port;      /* port to listen on */
  char *host;    /* hostmask to allow */
  int socket;    /* socket file descriptor */
  int type;      /* type of connections to accept */
  int tries;     /* how many times we've tried to bind the port */
};

struct Exceptlist
{
  struct Exceptlist *next;
  char *hostname;    /* hostname of user */
};

/*
 * Prototypes
 */

int Rehash();
int IsProtectedHost(char *hostname);
void ParseConf(char *filename, int rehash);
void LoadConfig();

struct Servlist *IsServLine(char *name, char *port);
struct PortInfo *IsListening(int port);
struct PortInfo *IsPort(int port);


/*
 * External declarations
 */

extern struct Servlist            *ServList;
extern struct PortInfo            *PortList;
extern struct ExceptionList       *ExceptList;
extern struct Servlist            *currenthub;

extern int                        HubCount;

#endif /* INCLUDED_conf_h */
