/*
 * settings.h
 * Copyright (C) 2000 Nedelcho Stanev
 *
 * $Id: settings.h,v 1.00 2000/03/03 22:17:23 decho Exp $
 */

#ifndef INCLUDED_settings_h
#define INCLUDED_settings_h

/* Parameter types */
#define   PARAM_STRING      1 /* parameter is a string */
#define   PARAM_INT         2 /* parameter is an integer */
#define   PARAM_TIME        3 /* parameter is in time format */
#define   PARAM_SET         4 /* parameter is configurable on/off */
#define   PARAM_PORT        5 /* parameter is a port number 1..65535 */

/* Directive types */
#define   D_REQUIRED        1 /* directive is required */
#define   D_OPTIONAL        2 /* directive is optional */
#define   D_NORUNTIME       3 /* directive is required and not run-time configurable */

#define   PARAM_MAX         5 /* max parameters on one line */

struct Directive
{
  char *name; /* directive name */
  int flag; /* directive flag - D_* */

  struct
  {
    int type; /* type of variable "ptr" is - PARAM_* */
    void *ptr; /* where to store parameter */
  } param[PARAM_MAX];
};

/*
 * Prototypes
 */

struct Directive *FindDirective(char *);
int LoadSettings(int rehash);
int SaveSettings();

/*
 * External declarations
 */

extern  struct Directive directives[];

extern  char      *HPath;
extern  char      *ConfigFile;
extern  char      *LogFile;
extern  char      *PidFile;
extern  char      *GlineReason;
extern  char      *KillReason;
extern  char      *CleanerName;
extern  char      *MonitoringChannel;
extern  char      *CheckNotifyMsg;

extern  long      ConnectFrequency;
extern  long      ConnectBurst;
extern  long      ServerBurst;
extern  int       ProxyCheck;
extern  int       MaxProxies;
extern  int       KillSocks;
extern  int       GlineSocks;
extern  int       EnableMonitoring;

extern  int       LogLevel;
extern  int       MaxLogs;

#ifdef USE_SERVICES
extern  char      *ServicesHost;
#endif

#endif /* INCLUDED_settings_h */
