/*
 * SocksCleaner TS Services, Copyright (C) 2000 Nedelcho Stanev
 * This program comes with absolutely NO WARRANTY
 *
 * Should you choose to use and/or modify this source code, please
 * do so under the terms of the GNU General Public License under which
 * this program is distributed.
 *
 * $Id: alloc.c,v 1.0.0 2000/03/03 22:17:23 decho Exp $
 */

#include <stdlib.h>
#include <sys/types.h>
#include <string.h>

#include "alloc.h"
#include "config.h"
#include "scdefs.h"
#include "log.h"
#include "misc.h"
#include "server.h"

/*
MyMalloc()
  attempt to malloc 'bytes' bytes of memory; returns a pointer
to allocated memory if successful, otherwise calls OutOfMem()
*/

void *
MyMalloc(size_t  bytes)

{
  void *ptr;

  ptr = (void *) malloc(bytes);
  if (!ptr)
    OutOfMem();

  return (ptr);
} /* MyMalloc() */

/*
MyRealloc()
  calls realloc() with 'oldptr' and 'newsize'; returns pointer to
allocated memory if successful, otherwise calls OutOfMem()
*/

void *
MyRealloc(void *oldptr, size_t newsize)

{
  void *ptr;

  ptr = (void *) realloc(oldptr, newsize);
  if (!ptr)
    OutOfMem();

  return (ptr);
} /* MyRealloc() */

/*
MyStrdup()
  duplicates the string 'str' and returns a pointer to the new
string
*/

char *
MyStrdup(const char *str)

{
  char *newstr;

  newstr = strdup(str);
  if (!newstr)
    OutOfMem();

  return (newstr);
} /* MyStrdup() */

/*
MyFree()
  Free memory pointed to by 'ptr'
NOTE: This is now a macro in hybdefs.h
*/

#if 0

void
MyFree(void *ptr)

{
  if (!ptr)
    return;

  free(ptr);
} /* MyFree() */

#endif /* 0 */

/*
OutOfMem()
  Called when a memory allocation attempt has failed - make a log
entry and a core file to work with
*/

void
OutOfMem()

{
  putlog(LOG1, "Out of Memory, shutting down");
  DoShutdown((char *) NULL, "Out of Memory");
} /* OutOfMem() */

