/*
 * SocksCleaner TS Services, Copyright (C) 2000 Nedelcho Stanev
 * This program comes with absolutely NO WARRANTY
 *
 * Should you choose to use and/or modify this source code, please
 * do so under the terms of the GNU General Public License under which
 * this program is distributed.
 *
 * $Id: proxy.c,v 1.0.0 2000/03/03 22:17:23 decho Exp $
 */

#include <stdio.h>
#include <unistd.h>
#include <assert.h>
#include <errno.h>
#include <string.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>

#include "client.h"
#include "config.h"
#include "defs.h"
#include "log.h"
#include "proxy.h"
#include "settings.h"
#include "sock.h"
#include "scdefs.h"
#include "server.h"
#include "misc.h"

//#define PTH_EXIT
//#define WITH_FORKS

void ExecuteGline( struct Luser* lptr, int type, int code );

/*
 * How many proxy queries we are currently engaged in -
 * If this exceeds MaxProxies, we stop doing proxy queries
 * for a bit. The idea is if a server netbursts, we don't
 * want a thousand proxy threads running.
 */
int ProxyHits = 0;

/*
Proxy()
 This routine will connect to the hostname that "nickname" is
using at SOCKSPORT. If it finds the "no authentication required" reply
assume the user is using an insecure SOCKS and notify +y users.
Autogline the host (if required).
*/

#define SOCKSPORT 1080
#define PROXYPORT   23

typedef struct SocksReply {
    unsigned char ver;
    unsigned char method;
  } SOCKS_REPLY;

void
Proxy(struct Luser* user)

{
  int sockfd, ret,bad;
  SOCKS_REPLY  pSocksReply;
  struct Luser lptr;
  
  iChecked++;
  iThreads++;
  bad = 0;
  assert(user != 0);

// First check for WinGates

#ifdef WITH_FORKS
  fork();
#endif

  memset (&lptr,0,sizeof(struct Luser));
  memcpy(&lptr,user,sizeof(struct Luser) );

  if( (strlen( lptr.hostname) == 0 ) || (strlen( lptr.nick) == 0) ) {
    iThreads--;
#ifndef  PTH_EXIT
    return ;
#else
   pthread_exit();
#endif
  }

  if( CheckNotifyMsg )
    notice(CleanerName, lptr.nick,CheckNotifyMsg );

  if ((sockfd = ConnectHost(lptr.hostname, SOCKSPORT)) >= 0)
  {
    /*
     * Connection successful, wait for write-ready
     */

     int soerror;
     socklen_t soerrlen;
     fd_set fds;
     struct timeval tv;

     FD_ZERO(&fds);        tv.tv_sec = 15;      /* 5 secs timeout */
     FD_SET(sockfd, &fds); tv.tv_usec = 0;

     if (!select(sockfd+1, NULL, &fds, NULL, &tv)) {
       close(sockfd);
       iThreads--;
#ifndef PTH_EXIT
       return;  
#else  
        pthread_exit();
#endif
     }
     soerrlen = sizeof(int);	
     getsockopt(sockfd, SOL_SOCKET, SO_ERROR, &soerror, &soerrlen);
     if (soerror) {
       iThreads--;
       close(sockfd);
#ifdef PTH_EXIT
       pthread_exit();
#else
       return;
#endif
      }

      /*
       * Probe SOCKS5 server, 1 method, no authentication
       *      (SOCKS4 with bogus connect request)
       */
      
      write(sockfd, "\x05\x01\x00", 3);

      FD_ZERO(&fds);        tv.tv_sec = 15;      /* 5 secs timeout */
      FD_SET(sockfd, &fds); tv.tv_usec = 0;

      if (!select(sockfd+1, &fds, NULL, NULL, &tv)) {
       close(sockfd);
       iThreads--;
#ifdef PTH_EXIT
       pthread_exit();
#else
       return;
#endif
      }

      memset(&pSocksReply, 0x00, sizeof(SOCKS_REPLY) );
      if ((ret = read(sockfd, &pSocksReply, sizeof(SOCKS_REPLY))) < 0)
           {
             putlog(LOG2,
             "Socks error from host %s: %s",
             lptr.hostname,
             strerror(errno));
             close(sockfd);
#ifndef PTH_EXIT
             return;
#else
             pthread_exit();
#endif
           }


           /*
            * An open SOCKS5 server will return zero for authentication method
            *         SOCKS4 server will return 91 for bogus request passed
            */
       if (pSocksReply.ver == 5 && pSocksReply.method == 0) 
            bad = 1;
       if (pSocksReply.ver == 0 && pSocksReply.method == 91)
            bad = 1;
      close (sockfd);
    } /* if host connected */
    else {
      iThreads--;
#ifdef PTH_EXIT
      pthread_exit();
#else
      return;
#endif
    }   

   if( bad ) {
      putlog(LOG2,
               "Insecure SOCKS: %s!%s@%s ver:%d method:%d KillSocks:%d GlineSocks:%d",
               lptr.nick,
               lptr.username,
               lptr.hostname,
               pSocksReply.ver,
               pSocksReply.method,
               KillSocks,
               GlineSocks);
      ExecuteGline( &lptr, pSocksReply.ver, pSocksReply.method );
   }
   iThreads--;
#ifdef PTH_EXIT
   pthread_exit();
#else
   return;
#endif
} /* Proxy */

void ExecuteGline( struct Luser* lptr, int type, int code )
{

    char inbuf[MAXLINE];

    if( IsProtectedHost(lptr->hostname))  {
      putlog( LOG1, "Nick %s with host %s is protected, skipped!",
                lptr->nick,
                lptr->hostname );
      if( EnableMonitoring && strcasecmp(lptr->hostname,"gw-nas.triada.bg") ) {
           sprintf( inbuf, "Nick %s with host %s is protected, skipped!",
                  lptr->nick,
                  lptr->hostname );
           toserv(":%s PRIVMSG %s :%s\n",
                  CleanerName,
                  MonitoringChannel,
                  inbuf );
      } /* enable monitoring */
      iProtected++;
      toserv("PUBPRX %s\n",
           lptr->nick);
      return;
    } /* is host protected? */
    memset( &inbuf,0x00, MAXLINE );

         if( KillSocks > 0 ) {
                sprintf( inbuf,":%s KILL %s :%s!%s (Glined: %s (%s))\n",
                   CleanerName,
                   lptr->nick,
                   Me.name,
                   CleanerName,
                   KillReason,
                   CleanerName );
                toserv(inbuf); 
              }
#ifdef HYBRID_GLINES
              if( GlineSocks >0  ) {
		sprintf( inbuf, ":%s GLINE Scleaner1 scleaner1 sockscleaner1.com sockscleaner1.org %s %s :%s\n",
			Me.name,
			"*",
			lptr->hostname,
			GlineReason);
		toserv( inbuf );
		sprintf( inbuf, ":%s GLINE Scleaner2 scleaner2 sockscleaner2.com sockscleaner2.org %s %s :%s\n",
			Me.name,
			"*",
			lptr->hostname,
			GlineReason);
		toserv( inbuf );
		sprintf( inbuf, ":%s GLINE Scleaner3 scleaner3 sockscleaner3.com sockscleaner3.org %s %s :%s\n",
			Me.name,
			"*",
			lptr->hostname,
			GlineReason);
		toserv( inbuf );
              }
#endif        
	     if( EnableMonitoring ) {
        	 sprintf(inbuf,
              		"Insecure socks found on %s Version:%d Method:%d", 
               		lptr->hostname, type, code );
                 toserv( ":%s PRIVMSG %s :%s\n", CleanerName, MonitoringChannel, inbuf ); 
            }  
  iGlined ++;
  return;
} /* ExecuteGline() */


void ProxyLoop()
{

   _UserStr* tmpstr;
   struct Luser s_luser;

  for (;;)
  {
     /* now browse all record in list */
     for (tmpstr = userstruct; tmpstr; tmpstr = tmpstr->next) {
        memset( &s_luser,0x00, sizeof(s_luser) );
        strcpy(s_luser.nick, tmpstr->nick);
        strcpy(s_luser.hostname, tmpstr->userhost);
        putlog( LOG1,"Perform check for host %s = %s ", s_luser.hostname, tmpstr->userhost );
        Proxy( &s_luser );
        putlog( LOG1, "Need to delete %s from hostlis", tmpstr->userhost );
        check_del( tmpstr->userhost );
     }
  } /* end of dead loop */
}
