/*
 * SocksCleaner TS Services, Copyright (C) 2000 Nedelcho Stanev
 * This program comes with absolutely NO WARRANTY
 *
 * Should you choose to use and/or modify this source code, please
 * do so under the terms of the GNU General Public License under which
 * this program is distributed.
 *
 * $Id: scleaner.c,v 1.0.0 2000/03/03 22:17:23 decho Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <string.h>

#include "alloc.h"
#include "config.h"
#include "defs.h"
#include "scdefs.h"
#include "init.h"
#include "log.h"
#include "server.h"
#include "settings.h"
#include "sock.h"
#include "timer.h"
#include "timestr.h"
#include "proxy.h"

#ifdef HAVE_PTHREADS
#include <pthread.h>
#endif

/* unixtime of when services was started */
time_t                       TimeStarted;
time_t                       TimeLastConnected;

/* offset from UTC time */
long                         gmt_offset;

/* structure containing services' info */
struct MyInfo                Me;

/*
 * Set to 1 "ConnectBurst" seconds after a successful
 * hub connection, so we don't send bursts of client/clone
 * connections to +y users etc, during an initial connect.
 */
int                          SafeConnect = 0;
int                          SplitBurst  = 0;

int
main(int argc, char *argv[])

{
#ifndef DEBUGMODE
  int pid; /* pid of this process */
#endif
  FILE *pidfile; /* to write our pid */
  uid_t uid, /* real user id */
        euid; /* effective user id */
#ifdef HAVE_PTHREADS
  pthread_attr_t attr;
  pthread_t selectid;
  pthread_t timerid;
#ifdef METHOD2
  pthread_t clientid;
  pthread_attr_t attr2;
#endif

#endif

  TimeStarted = time(NULL);

  iGlined    = 0;
  iChecked   = 0;
  iProtected = 0;
  iThreads   = 0;

  fprintf(stderr,
    "\nSocksCleaner TS services version %s by Nedelcho Stanev (decho@iname.com)\n",
    hVersion);

  /*
   * Load SETPATH (settings.conf) - this must be done
   * before the config file is loaded, and before any
   * putlog() calls are made, since LogFile is specified
   * in settings.conf
   */
  if (LoadSettings(0) == 0)
  {
    fprintf(stderr, "Fatal errors encountered and logged parsing %s, exiting\n",
      SETPATH);
    return (0);
  }


  putlog(LOG1, "SocksCleaner TS services started");

  /*
   * Get the offset from GMT (London time)
   */
  gmt_offset = GetGMTOffset(TimeStarted);

#if 0
  tm_gmt = gmtime(&TimeStarted);
  gmt_offset = TimeStarted - mktime(tm_gmt);
#endif /* 0 */

  uid = getuid(); /* the user id of the user who ran the process */
  euid = geteuid(); /* the effective id (different if setuid) */

  if (!uid || !euid)
  {
    /*
     * It is setuid root or running as root
     */
    printf("Please do not run SocksCleaner as root\n");
    putlog(LOG1, "Detected running with root uid, exiting");
    exit (0);
  }

  if (chdir(HPath) != 0)
  {
    fprintf(stderr,
      "HPath is an invalid directory, please check %s\n",
      SETPATH);
    putlog(LOG1, "Invalid HPath (%s), shutting down", HPath);
    exit(0);
  }

  /* load server, jupe, gline, user, admin info */
  LoadConfig(ConfigFile);


  if (LocalHostName)
    SetupVirtualHost();

#ifndef DEBUGMODE

  pid = fork();
  if (pid == (-1))
  {
    printf("Unable to fork(), exiting.\n");
    exit(0);
  }
  if (pid != 0)
  {
    printf("Running in background (pid: %d)\n\n", pid);
    exit(0);
  }
  close(0);
  close(1);
  close(2);

#else

  printf("Entering debug mode\n");  

#endif /* !DEBUGMODE */


  /*
   * Signals must be set up after fork(), since the parent
   * exits
   */
  InitSignals();

  srandom(time(NULL));

  /* write our pid to a file */
  if ((pidfile = fopen(PidFile, "w")) == (FILE *) NULL)
    putlog(LOG1, "Unable to open %s", PidFile);
  else
  {
    fprintf(pidfile, "%i\n", (int) getpid());
    fclose(pidfile);
  }

  HubSock = NOSOCKET;
  CycleServers();
  putlog( LOG1, "back , HubSock=%d", HubSock);
#ifdef HAVE_PTHREADS

  /*
   * This thread will call DoTimer() every second
   */
  pthread_attr_init(&attr);
  pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
  pthread_create(&timerid, &attr, p_CheckTime, NULL);

#ifdef METHOD2
  /* 
   * This will be run loop for checks
   */
   pthread_attr_init(&attr2);
   pthread_create(&clientid, NULL, (void *) &ProxyLoop, NULL );
   pthread_detach(clientid);
   pthread_join(clientid,NULL);
   
#endif

#endif

#ifdef DEBUGMODE2
  putlog( LOG1, "enterint in while!");
#endif

  while (1)
  {
    /* enter loop waiting for server info */

  #ifdef HAVE_PTHREADS
    pthread_create(&selectid, NULL, (void *) &ReadSocketInfo, NULL);
    pthread_join(selectid, NULL);
  #else

    ReadSocketInfo();

  #endif /* HAVE_PTHREADS */

    if (currenthub)
      if (currenthub->realname)
      {
        MyFree(currenthub->realname);
        currenthub->realname = (char *) NULL;
      }
    
    close(HubSock); /* There was an error */
    HubSock = NOSOCKET;
    currenthub->connect_ts = 0;

    /* 
     * whenever SocksCleaner connects/reconnects to a server, clear
     * users, servers, and chans
     */
    /*
     * ClearHashes() must be called AFTER ClearUsers(),
     * or StatServ's unique client counts will be off since
     * cloneTable[] would be NULL while it was trying to find
     * clones
     */
    putlog( LOG1, "--1--");
    PostCleanup();
  } /* while (1) */

  return (0);
} /* main() */
