/*
 * SocksCleaner TS Services, Copyright (C) 2000 Nedelcho Stanev
 * This program comes with absolutely NO WARRANTY
 *
 * Should you choose to use and/or modify this source code, please
 * do so under the terms of the GNU General Public License under which
 * this program is distributed.
 *
 * $Id: timer.c,v 1.0.0 2000/03/03 22:17:23 decho Exp $
 */

#include <unistd.h>
#include <sys/types.h>
#include <signal.h>

#include "config.h"
#include "defs.h"
#include "scdefs.h"
#include "init.h"
#include "log.h"
#include "settings.h"
#include "sock.h"
#include "timer.h"

#ifdef HAVE_PTHREADS
#include <pthread.h>
#endif

#ifdef HAVE_PTHREADS

/*
 * 1 if ok to call DoTimer()
 */
int               GoodTimer = 1;

#endif

#ifdef HAVE_PTHREADS

/*
p_CheckSignals()
 This thread will continually wait for various signals, and call
ProcessSignal() appropriately
*/

void *
p_CheckSignals()

{
  sigset_t set;
  int caught;

  sigemptyset(&set);
  sigaddset(&set, SIGINT);
  sigaddset(&set, SIGHUP);
  sigaddset(&set, SIGSEGV);
  sigaddset(&set, SIGBUS);
  sigaddset(&set, SIGTERM);
  sigaddset(&set, SIGCHLD);
  sigaddset(&set, SIGPIPE);

  pthread_sigmask(SIG_BLOCK, &set, NULL);

  while (1)
  {
    sigwait(&set, &caught); /* wait until we get a signal */
    ProcessSignal(caught); /* we got a signal, process it */
  }

  return ((void *) NULL);
} /* p_CheckSignals() */

/*
p_CheckTime()
 This function will be called from a new thread to enter
an infinite loop and call DoTimer() once every second
*/

void *
p_CheckTime()

{
  time_t current_ts = time(NULL);

  while (1)
  {
    sleep(1);
    if (GoodTimer)
      DoTimer(current_ts);

    /*
     * Every five minutes, reset current_ts to time() in case a
     * DoTimer() call took over a second, in which case current_ts
     * will be a little off
     */
    if ((current_ts % 300) == 0)
      current_ts = time(NULL);
    else
      current_ts++;
  }

  return ((void *) NULL);
} /* p_CheckTime() */

#endif /* HAVE_PTHREADS */

/*
DoTimer()
  Execute various commands every second - such as checking for 
expired temp glines etc.
*/

void
DoTimer(time_t unixtime)

{

  if ((HubSock == NOSOCKET) && ((unixtime % ConnectFrequency) == 0)) {
    CycleServers();
    SafeConnect = 0;
  }
  if ((unixtime % 3600) == 0)
  {
    /*
     * Check for expired nicknames, channels, and memos every hour
     */

  } /* if ((unixtime % 3600) == 0) */


  /*
   * we have to add gmt_offset here or it won't be true at
   * midnight unless we are in the GMT timezone - for instance,
   * if we are in the EST timezone, we would be 5 hours earlier
   * than GMT, and thus ((unixtime % 86400) == 0) would only
   * be true at 19:00, so if we add gmt_offset (which would
   * be negative) to unixtime (at midnight), the value would
   * be the TS of 19:00, which would return true
   */
  if (((unixtime + gmt_offset) % 86400) == 0)
  {

    /*
     * every day at midnight backup the log file
     * to the format: logfile.YYYYMMDD and start
     * a new one.
     */
    CheckLogs(unixtime);
  } /* if (((unixtime + gmt_offset) % 86400) == 0) */

  /*
   * If SendUmode() detected a "function-calling" flood,
   * check if 20 seconds have passed; if so, remove flood
   * protection
   */

    if (currenthub)
    {
      if (!currenthub->next)
        currenthub = ServList;
      else
        currenthub = currenthub->next;
    }
    else
      currenthub = ServList;

  if (!SafeConnect && currenthub)
  {
    /*
     * currenthub->connect_ts + ConnectBurst is the TS
     * of when we can assume we are safely connected to
     * the network - that is, there is no more initial
     * connect burst information coming. So, we can now
     * broadcast clone/client connections to +y users
     * without fearing they will get flooded with hundreds
     * of them from an initial burst.
     * Make sure currenthub->realname is not null, so we
     * can tell if we are actually connected to a hub :-)
     */
    if ( /* currenthub->realname && */
        (unixtime >= (currenthub->connect_ts + ConnectBurst))) {
      SafeConnect = 1;
      putlog( LOG1, "Entering in safe connect mode!");
      if( EnableMonitoring ) 
         toserv(":%s PRIVMSG %s :%s\n",
              CleanerName,
              MonitoringChannel,
              "Entering in safe mode.");

    }
  } /* if (!SafeConnect && currenthub) */

  if( !SplitBurst ) {

    if( unixtime >= (TimeLastConnected + ServerBurst ) ) {
        SplitBurst = 1;

       if( EnableMonitoring ) {
         toserv(":%s PRIVMSG %s :%s\n",
              CleanerName,
              MonitoringChannel,
              "Resuming standart opertation.");      
       } /* Enable monitoring == 1 */
    } /* if( unixtime >= ... */
  } /* SplitBurst == 0 */

} /* DoTimer() */
